//
//  CommunityMemberViewModel.swift
//  SampleApp
//
//  Created by Nishan Niraula on 12/16/20.
//  Copyright © 2020 David Zhang. All rights reserved.
//

import Foundation
import AmitySDK

struct CommunityMember: Identifiable {
    let id: UUID
    let memberUserId: String
    let displayName: String
    var roles: String = ""
    let isBanned: Bool
    
    init(member: AmityCommunityMember) {
        id = UUID()
        memberUserId = member.userId
        displayName = member.displayName
        isBanned = member.isBanned
        if !member.roles.isEmpty {
            roles = member.roles.joined(separator: ",")
        }
    }
    
    init(member: AmityUser) {
        id = UUID()
        memberUserId = member.userId
        displayName = member.displayName ?? member.userId
        isBanned = false
        if !member.roles.isEmpty {
            roles = member.roles.joined(separator: ",")
        }
    }
}

class CommunityMemberViewModel: ObservableObject {
    
    let communityId: String
    let commMembership: AmityCommunityMembership
    let commModeration: AmityCommunityModeration
    let commRepository: AmityCommunityRepository
    let membership: [String] = ["none", "member", "banned"]
    let sortBy: [String] = ["lastCreated", "firstCreated"]
    
    var token: AmityNotificationToken?
    var roleToFilter: String = ""
    
    @Published var roleUpdateStatus = ""
    @Published var banUnbanUpdateStatus = ""
    @Published var permissionStatus = ""
    @Published var members: [CommunityMember] = []
    
    init(communityId: String) {
        self.communityId = communityId
        self.commMembership = AmityCommunityMembership(client: AmityManager.shared.client!, andCommunityId: communityId)
        self.commModeration = AmityCommunityModeration(client: AmityManager.shared.client!, andCommunity: communityId)
        self.commRepository = AmityCommunityRepository(client: AmityManager.shared.client!)
    }
    
    func addRoles(roles: [String], userId: String) {
        Task { @MainActor in
            var success: Bool = false
            do {
                let result = try await self.commModeration.addRoles(roles, userIds: [userId])
                success = result
            } catch {}
            self.roleUpdateStatus = success ? "Role Added" : "Error"
        }
    }
    
    func removeRoles(roles: [String], userId: String) {
        Task { @MainActor in
            var success: Bool = false
            do {
                let result = try await self.commModeration.removeRoles(roles, userIds: [userId])
                success = result
            } catch {}
            self.roleUpdateStatus = success ? "Role Removed" : "Error"
        }
    }
    
    func checkPermission(permission: AmityPermission) {
        AmityManager.shared.client?.hasPermission(permission, forCommunity: communityId, completion: { [weak self] hasPermission in
            self?.permissionStatus = hasPermission ? "Permission: True" : "Permission: False"
        })
    }
    
    func banUser(userId: [String]) {
        Task { @MainActor in
            var success: Bool = false
            do {
                let result = try await self.commModeration.banMembers(userId)
                success = result
            } catch {}
            self.banUnbanUpdateStatus = success ? "Ban Success" : "Ban Error"
        }
    }
    
    func unBan(userId: [String]) {
        Task { @MainActor in
            var success: Bool = false
            do {
                let result = try await self.commModeration.unbanMembers(userId)
                success = result
            } catch {}
            self.banUnbanUpdateStatus = success ? "UnBan Success" : "Unban Error"
        }
    }
    
    func fetchMembers(keyword: String, membership membershipOptions: AmityCommunityMembershipFilterOption, roles: [String], sortBy: AmityCommunityMembershipSortOption) {
        self.members = []
        self.token = self.commMembership.searchMembers(keyword: keyword, membership: membershipOptions, roles: roles, sortBy: sortBy).observe({ [weak self] (collection, _, error) in
            Log.add(info: "----- Community Member Observer Triggered ----- ")
            Log.add(info: "Data Status: \(collection.dataStatus.description)")
            Log.add(info: "Collection Count: \(collection.count())")
            Log.add(info: "Collection All Objects Count: \(collection.allObjects().count)")
            
            var list = [CommunityMember]()
            for member in collection.allObjects() {
                let model = CommunityMember(member: member)
                list.append(model)
            }
            
            list.forEach { member in
                Log.add(info: "Member: \(member.displayName) Id: \(member.memberUserId)")
            }
            
            self?.members = list
        })
    }
    
//    func fetchMembers(roles: [String]) {
//        self.token = self.commParticipation.getMembers(membershipOptions: [.member], roles: roles, sortBy: .lastCreated).observe({ [weak self] (collection,_, error) in
//
//            Log.add(info: "----- Community Member Observer Triggered ----- ")
//            Log.add(info: "Data Status: \(collection.dataStatus.description)")
//            Log.add(info: "Collection Count: \(collection.count())")
//            Log.add(info: "Collection All Objects Count: \(collection.allObjects().count)")
//
//            var list = [CommunityMember]()
//            for member in collection.allObjects() {
//                let model = CommunityMember(member: member)
//                list.append(model)
//            }
//
//            list.forEach { member in
//                Log.add(info: "Member: \(member.displayName) Id: \(member.memberUserId)")
//            }
//
//            self?.members = list
//        })
//    }
    
    func resetStatus() {
        roleUpdateStatus = ""
        permissionStatus = ""
    }
}

extension AmityPermission {
    
    var identifier: String {
        switch (self) {
        case .muteChannel:
            return "MUTE CHANNEL"
        case .closeChannel:
            return "CLOSE CHANNEL"
        case .editChannel:
            return "EDIT CHANNEL"
        case .editChannelRateLimit:
            return "EDIT CHANNEL RATELIMIT"
        case .editMessage:
            return "EDIT MESSAGE"
        case .deleteMessage:
            return "DELETE MESSAGE"
        case .banChannelUser:
            return "BAN CHANNEL USER"
        case .muteChannelUser:
            return "MUTE CHANNEL USER"
        case .addChannelUser:
            return "ADD CHANNEL USER"
        case .removeChannelUser:
            return "REMOVE CHANNEL USER"
        case .editChannelUser:
            return "EDIT CHANNEL USER"
        case .banUser:
            return "BAN USER"
        case .editUser:
            return "EDIT USER"
        case .assignUserRole:
            return "ASSIGN USER ROLE"
        case .editUserFeedPost:
            return "EDIT USER FEED POST"
        case .deleteUserFeedPost:
            return "DELETE USER FEED POST"
        case .editUserFeedComment:
            return "EDIT USER FEED COMMENT"
        case .deleteUserFeedComment:
            return "DELETE USER FEED COMMENT"
        case .addCommunityUser:
            return "ADD COMMUNITY USER"
        case .removeCommunityUser:
            return "REMOVE COMMUNITY USER"
        case .editCommunityUser:
            return "EDIT COMMUNITY USER"
        case .banCommunityUser:
            return "BAN COMMUNITY USER"
        case .muteCommunityUser:
            return "MUTE COMMUNITY USER"
        case .editCommunity:
            return "EDIT COMMUNITY"
        case .deleteCommunity:
            return "DELETE COMMUNITY"
        case .editCommunityPost:
            return "EDIT COMMUNITY POST"
        case .deleteCommunityPost:
            return "DELETE COMMUNITY POST"
        case .pinCommunityPost:
            return "PIN COMMUNITY POST"
        case .editCommunityComment:
            return "EDIT COMMUNITY COMMENT"
        case .deleteCommunityComment:
            return "DELETE COMMUNITY COMMENT"
        case .reviewCommunityPost:
            return "REVIEW COMMUNITY POST"
        case .createCommunityPost:
            return "CREATE COMMUNITY POST"
        default:
            return ""
        }
        
    }
}
