//
//  CommunityView.swift
//  SampleApp
//
//  Created by Michael Abadi on 10/07/20.
//  Copyright © 2020 David Zhang. All rights reserved.
//

import SwiftUI
import AmitySDK

struct CommunityView: View {
    
    @ObservedObject var viewModel: CommunityListViewModel
    
    @Environment(\.navigationController) var navigationController
    
    @State private var memberType = AmityCommunityQueryFilter.all
    @State private var sortType = AmityCommunitySortOption.lastCreated
    @State private var includeDeleted = false
    @State private var categoryId = ""
    
    @State private var showCategoryListScreen = false
    @State private var showCreateScreen = false
    @State private var forceScrollToTop = false
    @State private var selection: String?
    
    var body: some View {
        VStack {
            if viewModel.type == .normal {
                // NOTE: Only show the query menu for `.normal`.
                // The user need to press "Get Communities" button manually.
                // For other types, the communities will be queried automatically.
                // See Also: .onAppear method in this view.
                communitiesQueryMenu()
            }
            // Communities List
            List(viewModel.community) { community in
                communityView(community)
            }
        }
        .navigationBarTitle(Text(viewModel.pageTitle), displayMode: .inline)
        .toolbar {
            ToolbarItemGroup(placement: .navigationBarTrailing) {
                // Categories list
                Button(action: {
                    self.showCategoryListScreen = true
                }, label: {
                    Image(systemName: "list.bullet")
                })
                .sheet(isPresented: $showCategoryListScreen) {
                    CommunityCategoryListView(viewModel: self.viewModel)
                }
                // Create a new community
                NavigationLink(destination: CommunityEditorView(viewModel: CommunityEditorViewModel()), isActive: $showCreateScreen) {
                    Button(action: {
                        self.showCreateScreen = true
                    }, label: {
                        Image(systemName: "plus")
                    })
                }
            }
        }
        .onAppear(perform: {
            selection = nil
            showCreateScreen = false
            if viewModel.type != .normal {
                // Automatically query communities for type orther than `.normal`.
                viewModel.queryCommunity()
            }
            UITableViewCell.appearance().selectionStyle = .none
            UITableView.appearance().separatorStyle = .none
            UITextField.appearance().clearButtonMode = .always
        })
        
    }
    
    func communitiesQueryMenu() -> some View {
        Group {
            // Member Type
            HStack {
                Text("Member Type").font(.caption)
                Spacer()
                Picker("Member Type", selection: $memberType) {
                    Text("All").tag(AmityCommunityQueryFilter.all)
                    Text("Member").tag(AmityCommunityQueryFilter.userIsMember)
                    Text("Not Member").tag(AmityCommunityQueryFilter.userIsNotMember)
                }
                .pickerStyle(.segmented)
            }
            .padding([.leading, .trailing])
            // Sort Type
            HStack {
                Text("Sort By").font(.caption)
                Spacer()
                Picker("Sort Type", selection: $sortType) {
                    Text("Last created").tag(AmityCommunitySortOption.lastCreated)
                    Text("First created").tag(AmityCommunitySortOption.firstCreated)
                    Text("Display Name").tag(AmityCommunitySortOption.displayName)
                }
                .pickerStyle(.segmented)
            }
            .padding([.leading, .trailing])
            // Include Deleted
            HStack {
                Text("Include Deleted").font(.caption)
                Spacer()
                Picker("isDeleted", selection: $includeDeleted) {
                    Text("NO").tag(false)
                    Text("YES").tag(true)
                }.pickerStyle(.segmented)
            }
            .padding([.leading, .trailing])
            // Category Id
            HStack {
                Text("CategoryId").font(.caption)
                Spacer()
                TextField("Search by category id", text: $categoryId)
                    .textFieldStyle(RoundedBorderTextFieldStyle())
            }
            .padding([.leading, .trailing])
            // Search Field
            HStack {
                Text("Keyword").font(.caption)
                Spacer()
                TextField("Search by display name", text: $viewModel.searchKeyword)
                    .textFieldStyle(RoundedBorderTextFieldStyle())
            }
            .padding([.leading, .trailing])
            // Query Button
            HStack {
                Spacer()
                let button = Button("Get Communities") {
                    forceScrollToTop.toggle()
                    viewModel.setFilter(memberType)
                    viewModel.setSort(sortType)
                    viewModel.setIncludeDeleted(includeDeleted)
                    viewModel.setCategoryId(categoryId)
                    viewModel.queryCommunity()
                }
                if #available(iOS 15.0, *) {
                    button.buttonStyle(.bordered)
                } else {
                    button.buttonStyle(.borderless)
                }
            }
            .padding([.leading, .trailing])
        }
    }
    
    func communityView(_ community: CommunityListModel) -> some View {
        ZStack {
            CardView(model: community, joinAction: {
                self.viewModel.joinCommunity(for: community, completion: nil)
            }, leaveAction: {
                self.viewModel.leaveCommunity(for: community, completion: nil)
            }, deleteAction: {
                self.viewModel.deleteCommunity(for: community, completion: nil)
            }, detailsAction: {
                self.selection = community.id + "+detail"
            }, updateAction: {
                self.selection = community.id + "+edit"
            }, membershipAction: {
                self.selection = community.id + "+participation"
            }, feedAction: {
                self.selection = community.id + "+feed"
                if selection == (community.id + "+feed") {
                    let feedManager = UserPostsFeedManager(client: AmityManager.shared.client!, userId: nil, userName: nil, community: community)
                    feedManager.feedType = .community
                    feedManager.community = community
                    
                    let postsFeedStoryboard = UIStoryboard(name: "Feed", bundle: nil)
                    let postsFeedController = postsFeedStoryboard.instantiateViewController(withIdentifier: UserPostsFeedViewController.identifier) as! UserPostsFeedViewController
                    postsFeedController.feedManager = feedManager
                    self.navigationController?.pushViewController(postsFeedController, animated: true)
                }
            }, realTimeEventAction: {
                let eventController = CommunityRealTimeEventController()
                let manager = CommunityRealTimeEventManager(community: community.communityObject)
                eventController.manager = manager
                
                self.navigationController?.pushViewController(eventController, animated: true)
            })
            .onAppear {
                if self.viewModel.community.last == community {
                    self.viewModel.fetchNextPage()
                }
            }
            
            // Update Navigattion
            let editorView = CommunityEditorView(viewModel: CommunityEditorViewModel(draft: CommunityDraft(community: community)))
            
            NavigationLink(
                destination: editorView,
                tag: community.id + "+edit",
                selection: self.$selection,
                label: {
                    EmptyView()
                })
            .buttonStyle(PlainButtonStyle())
            .hidden()
            
            // Participation Navigation
            let participationModel = CommunityParticipationViewModel(membership: community.getCommunityObject().membership)
            NavigationLink(
                destination: CommunityParticipationView(viewModel: participationModel),
                tag: community.id + "+participation",
                selection: self.$selection,
                label: {
                    EmptyView()
                })
            .buttonStyle(PlainButtonStyle())
            .hidden()
            
            // Details
            NavigationLink(
                destination: CommunityDetailView(viewModel: CommunityDetailViewModel(community: community)),
                tag: community.id + "+detail",
                selection: self.$selection,
                label: {
                    EmptyView()
                })
            .buttonStyle(PlainButtonStyle())
            .hidden()
        }
    }
        
}

struct CommunitiesView_Previews: PreviewProvider {
    
    static var previews: some View {
        return Text("Community View")
    }
    
}

struct CardTextView: View {
    
    let key: String
    let value: String
    
    var body: some View {
        HStack(alignment: .top) {
            Text(key)
                .padding(.leading, 20)
                .foregroundColor(Color.white)
                .font(.system(size: 15))
            
            Text(value)
                .foregroundColor(Color.white.opacity(0.8))
                .font(.system(size: 15))
                .lineLimit(1)
            
            Spacer()
        }
    }
    
}

struct ActionButtonStyle: ViewModifier {
    
    func body(content: Content) -> some View {
        content
            .padding([.leading, .trailing], 8)
            .padding([.top, .bottom], 8)
            .background(Color.black)
            .cornerRadius(4)
            .foregroundColor(Color.white)
    }
    
}
