//
//  FilePicker.swift
//  SampleApp
//
//  Created by Nontapat Siengsanor on 27/4/2565 BE.
//  Copyright © 2565 BE David Zhang. All rights reserved.
//

import UIKit
import MobileCoreServices

protocol FilePickerDelegate: AnyObject {
    func didPickFiles(filesUrl: [URL], documentTypes: [String])
}

extension URL {
    var isDirectory: Bool {
        return (try? resourceValues(forKeys: [.isDirectoryKey]))?.isDirectory ?? false
    }
}

open class FilePicker: NSObject {
    private var pickerController: UIDocumentPickerViewController?
    private weak var presentationController: UIViewController?
    private weak var delegate: FilePickerDelegate?
    private var filesUrl = [URL]()
    
    var numberOfIFiles: Int = 10
    var maximumSizeOfIFiles: Int = 1_073_741_824 // 1GB
    private var documentTypes: [String] = []
    
    init(presentationController: UIViewController, delegate: FilePickerDelegate) {
        super.init()
        
        self.presentationController = presentationController
        self.delegate = delegate
    }
    
    public func present(from sourceView: UIView, documentTypes: [String] = [kUTTypeContent, kUTTypeArchive] as [String]) {
        self.documentTypes = documentTypes
        pickerController = UIDocumentPickerViewController(documentTypes: documentTypes, in: .import)
        pickerController?.modalPresentationStyle = .overFullScreen
        pickerController?.delegate = self
        presentationController?.present(pickerController!, animated: true)
        
    }
    
}

extension FilePicker: UIDocumentPickerDelegate{
    
    public func documentPicker(_ controller: UIDocumentPickerViewController, didPickDocumentsAt urls: [URL]) {
        
        let fileSizes = urls.compactMap { try? $0.resourceValues(forKeys:[.fileSizeKey]).fileSize }
        
        guard fileSizes.filter({ $0 > maximumSizeOfIFiles}).isEmpty else {
            let alertController = UIAlertController(title: "Unable to attached the file", message: "The selected file is larger than 1GB. Plese select a new file.", preferredStyle: .alert)
            let cancelAction = UIAlertAction(title: "OK", style: .cancel, handler: nil)
            alertController.addAction(cancelAction)
            presentationController?.present(alertController, animated: true, completion: nil)
            return
        }
        
        let sumNumberOfItems = filesUrl.count + urls.count
        guard sumNumberOfItems <= numberOfIFiles else {
            let alertController = UIAlertController(title: "Maximum number of files exceeded", message: "Maximum number of files that can be uploaded is \(numberOfIFiles). The rest files will be discarded.", preferredStyle: .alert)
            let cancelAction = UIAlertAction(title: "OK", style: .cancel, handler: nil)
            alertController.addAction(cancelAction)
            presentationController?.present(alertController, animated: true, completion: nil)
            return
        }
        
        for url in urls {
            documentFromURL(pickedURL: url)
        }
        
        delegate?.didPickFiles(filesUrl: filesUrl, documentTypes: documentTypes)
    }
    
    public func documentPickerWasCancelled(_ controller: UIDocumentPickerViewController) {
        // User cancelled
    }
    
    private func documentFromURL(pickedURL: URL) {
        NSFileCoordinator().coordinate(readingItemAt: pickedURL, error: NSErrorPointer.none) { (fileUrl) in
            filesUrl.append(fileUrl)
        }
        
        let shouldStopAccessing = pickedURL.startAccessingSecurityScopedResource()
        
        if shouldStopAccessing {
            pickedURL.stopAccessingSecurityScopedResource()
        }
    }
    
}

extension FilePicker: UINavigationControllerDelegate {}
